@@ -775,7 +775,7 @@ char *__pformat_cvt( int mode, __pformat_fpreg_t x, int nd, int *dp, int *sign )
    * replacements for `ecvt()' and `fcvt()'.
    */
   int k; unsigned int e = 0; char *ep;
-  static FPI fpi = { 64, 1-16383-64+1, 32766-16383-64+1, FPI_Round_near, 0 };
+  static FPI fpi = { 64, 1-16383-64+1, 32766-16383-64+1, FPI_Round_near, 0, 14 /* Int_max */ };
  
   /* Classify the argument into an appropriate `__gdtoa()' category...
    */
@@ -2301,19 +2301,9 @@ __pformat (int flags, void *dest, int max, const APICHAR *fmt, va_list argv)
 	       */
 	      length = PFORMAT_LENGTH_LONG;
 
-#           ifndef _WIN32
-	      /*
-	       * Microsoft's MSVCRT implementation also uses `l'
-	       * as a modifier for `long double'; if we don't want
-	       * to support that, we end this case here...
-	       */
 	      state = PFORMAT_END;
 	      break;
 
-	      /* otherwise, we simply fall through...
-	       */
-#	    endif
-
 	  case 'L':
 	    /*
 	     * Identify the appropriate argument as a `long double',
