@@ -70,12 +70,10 @@ void cond_print(volatile pthread_cond_t *c, char *txt)
     if (c_ == NULL) {
         fprintf(fo,"C%p %d %s\n",*c,(int)GetCurrentThreadId(),txt);
     } else {
-        fprintf(fo,"C%p %d V=%0X B=%d b=%p w=%ld %s\n",
+        fprintf(fo,"C%p %d V=%0X w=%ld %s\n",
             *c, 
             (int)GetCurrentThreadId(), 
             (int)c_->valid, 
-            (int)c_->busy,
-            NULL,
             c_->waiters_count_,
             txt
             );
@@ -209,7 +207,7 @@ pthread_cond_init (pthread_cond_t *c, const pthread_condattr_t *a)
       return ENOMEM; 
   }
   _c->valid  = DEAD_COND;
-
+  _c->busy = 0;
   _c->waiters_count_ = 0;
   _c->waiters_count_gone_ = 0;
   _c->waiters_count_unblock_ = 0;
@@ -275,7 +273,7 @@ pthread_cond_destroy (pthread_cond_t *c)
        do_sema_b_release (_c->sema_b, 1,&_c->waiters_b_lock_,&_c->value_b);
        return EBUSY;
     }
-  if (_c->waiters_count_ > _c->waiters_count_gone_ || _c->busy != 0)
+  if (_c->waiters_count_ > _c->waiters_count_gone_)
     {
       r = do_sema_b_release (_c->sema_b, 1,&_c->waiters_b_lock_,&_c->value_b);
       if (!r) r = EBUSY;
@@ -319,7 +317,7 @@ pthread_cond_signal (pthread_cond_t *c)
       if (_c->waiters_count_ == 0)
       {
 	LeaveCriticalSection (&_c->waiters_count_lock_);
-	pthread_testcancel();
+	/* pthread_testcancel(); */
 	return 0;
       }
       _c->waiters_count_ -= 1;
@@ -331,7 +329,7 @@ pthread_cond_signal (pthread_cond_t *c)
       if (r != 0)
       {
 	LeaveCriticalSection (&_c->waiters_count_lock_);
-	pthread_testcancel();
+	/* pthread_testcancel(); */
 	return r;
       }
       if (_c->waiters_count_gone_ != 0)
@@ -345,12 +343,12 @@ pthread_cond_signal (pthread_cond_t *c)
   else
     {
       LeaveCriticalSection (&_c->waiters_count_lock_);
-      pthread_testcancel();
+      /* pthread_testcancel(); */
       return 0;
     }
   LeaveCriticalSection (&_c->waiters_count_lock_);
   r = do_sema_b_release(_c->sema_q, 1,&_c->waiters_q_lock_,&_c->value_q);
-  pthread_testcancel();
+  /* pthread_testcancel(); */
   return r;
 }
 
@@ -376,7 +374,7 @@ pthread_cond_broadcast (pthread_cond_t *c)
       if (_c->waiters_count_ == 0)
       {
 	LeaveCriticalSection (&_c->waiters_count_lock_);
-	pthread_testcancel();
+	/* pthread_testcancel(); */
 	return 0;
       }
       relCnt = _c->waiters_count_;
@@ -389,7 +387,7 @@ pthread_cond_broadcast (pthread_cond_t *c)
       if (r != 0)
       {
 	LeaveCriticalSection (&_c->waiters_count_lock_);
-	pthread_testcancel();
+	/* pthread_testcancel(); */
 	return r;
       }
       if (_c->waiters_count_gone_ != 0)
@@ -404,12 +402,12 @@ pthread_cond_broadcast (pthread_cond_t *c)
   else
     {
       LeaveCriticalSection (&_c->waiters_count_lock_);
-      pthread_testcancel();
+      /* pthread_testcancel(); */
       return 0;
     }
   LeaveCriticalSection (&_c->waiters_count_lock_);
   r = do_sema_b_release(_c->sema_q, relCnt,&_c->waiters_q_lock_,&_c->value_q);
-  pthread_testcancel();
+  /* pthread_testcancel(); */
   return r;
 }
 
@@ -420,7 +418,7 @@ pthread_cond_wait (pthread_cond_t *c, pthread_mutex_t *external_mutex)
   cond_t *_c;
   int r;
 
-  pthread_testcancel();
+  /* pthread_testcancel(); */
 
   if (!c || *c == NULL)
     return EINVAL;
@@ -457,15 +455,15 @@ pthread_cond_wait (pthread_cond_t *c, pthread_mutex_t *external_mutex)
   return r;
 }
 
-int
-pthread_cond_timedwait (pthread_cond_t *c, pthread_mutex_t *external_mutex, const struct timespec *t)
+static int
+pthread_cond_timedwait_impl (pthread_cond_t *c, pthread_mutex_t *external_mutex, const struct timespec *t, int rel)
 {
   sCondWaitHelper ch;
   DWORD dwr;
   int r;
   cond_t *_c;
 
-  pthread_testcancel();
+  /* pthread_testcancel(); */
 
   if (!c || !*c)
     return EINVAL;
@@ -479,7 +477,15 @@ pthread_cond_timedwait (pthread_cond_t *c, pthread_mutex_t *external_mutex, cons
   } else if ((_c)->valid != (unsigned int)LIFE_COND)
     return EINVAL;
 
-  dwr = dwMilliSecs(_pthread_rel_time_in_ms(t));
+  if (rel == 0)
+  {
+    dwr = dwMilliSecs(_pthread_rel_time_in_ms(t));
+  }
+  else
+  {
+    dwr = dwMilliSecs(_pthread_time_in_ms_from_timespec(t));
+  }
+
   r = do_sema_b_wait (_c->sema_b, 0, INFINITE,&_c->waiters_b_lock_,&_c->value_b);
   if (r != 0)
     return r;
@@ -491,7 +497,7 @@ pthread_cond_timedwait (pthread_cond_t *c, pthread_mutex_t *external_mutex, cons
   ch.c = _c;
   ch.r = &r;
   ch.external_mutex = external_mutex;
-  { 
+  {
     pthread_cleanup_push(cleanup_wait, (void *) &ch);
 
     r = pthread_mutex_unlock(external_mutex);
@@ -503,6 +509,18 @@ pthread_cond_timedwait (pthread_cond_t *c, pthread_mutex_t *external_mutex, cons
   return r;
 }
 
+int
+pthread_cond_timedwait(pthread_cond_t *c, pthread_mutex_t *m, const struct timespec *t)
+{
+  return pthread_cond_timedwait_impl(c, m, t, 0);
+}
+
+int
+pthread_cond_timedwait_relative_np(pthread_cond_t *c, pthread_mutex_t *m, const struct timespec *t)
+{
+  return pthread_cond_timedwait_impl(c, m, t, 1);
+}
+
 static void
 cleanup_wait (void *arg)
 {
@@ -600,7 +618,7 @@ do_sema_b_wait_intern (HANDLE sema, int nointerrupt, DWORD timeout)
     return r;
   }
   arr[0] = sema;
-  arr[1] = (HANDLE) pthread_getevent (pthread_self ());
+  arr[1] = (HANDLE) pthread_getevent ();
   if (arr[1] != NULL) maxH += 1;
   if (maxH == 2)
   {
